<?php namespace EscapeWork\Manager\Model;

use EscapeWork\LaravelHelpers\Eloquent;
use EscapeWork\Manager\Repository\RoleActionRepository;
use EscapeWork\Manager\Repository\UserRepository;

class RoleAction extends Eloquent implements RoleActionRepository
{

    /**
     * Table
     */
    protected $table = 'manager_role_actions';

    /**
     * Fillable fields 
     */
    public $fillable = array(
        'role_id', 
        'action_id', 
        'page_id'
    );

    /**
     * The primary key for the model.
     * Not really the primary key, but is used in Illuminate\Database\Eloquent\Collection->contains() method
     *
     * @var string
     */
    protected $primaryKey = 'action_id';

    /**
     * Belongs to Page
     */
    public function page()
    {
        return $this->belongsTo('EscapeWork\Manager\Model\Page');
    }

    /**
     * Belongs to Role
     */
    public function role()
    {
        return $this->belongsTo('EscapeWork\Manager\Model\Role');
    }

    /**
     * Belongs to PageAction
     */
    public function action()
    {
        return $this->belongsTo('EscapeWork\Manager\Model\PageAction', 'action_id');
    }

    /**
     * Saving all actions from a role
     */
    public function saveAll($actions, $pages)
    {
        foreach ($actions as $action_id) {
            $page_id = $pages[$action_id];

            $this->insertNew($this->role_id, $action_id, $page_id);
        }
    }

    /**
     * Criando uma nova ação
     */
    public function insertNew($role_id, $action_id, $page_id)
    {
        $roleAction            = new RoleAction();
        $roleAction->role_id   = $role_id;
        $roleAction->action_id = $action_id;
        $roleAction->page_id   = $page_id;
        $roleAction->save();
    }

    /**
     * Destroying all actions in a role
     */
    public function deleteByRole($role_id)
    {
        $this->where('role_id', '=', $role_id)->delete();
    }

    /**
     * Destroying all role actions in a specific page
     */
    public function deleteByPage($page_id)
    {
        $this->where('page_id', '=', $page_id)->delete();
    }

    /**
     * Getting all actions in a role
     */
    public function getByRole($role_id)
    {
        return $this->where('role_id', '=', $role_id)->get();
    }

    /**
     * Scope for join with the users table
     */
    public function scopeJoinUser($query)
    {
        return $query->join('manager_users', 'manager_users.role_id', '=', 'manager_role_actions.role_id');
    }

    /**
     * Scope for join with the pages table
     */
    public function scopeJoinPage($query)
    {
        return $query->join('manager_pages', 'manager_pages.id', '=', 'manager_role_actions.page_id');
    }

    /**
     * Scope for join with the actions table
     */
    public function scopeJoinAction($query)
    {
        return $query->join('manager_page_actions', 'manager_page_actions.id', '=', 'manager_role_actions.action_id');
    }

    /**
     * Checking if some user has permission to do some action in some page
     *
     * @param  UserRepository $user
     * @param  string         $page
     * @param  string         $action
     */
    public function hasPermission(UserRepository $user, $page, $action)
    {
        return $this->select('manager_users.id')
             ->joinUser()
             ->joinPage()
             ->joinAction()
             ->where('manager_users.id', '=', $user->id)
             ->where('manager_pages.slug', '=', $page)
             ->where('manager_page_actions.type', '=', $action)
             ->first();
    }

    /**
     * Fetching all the user actions
     *
     * @access  public
     * @param   UserRepository $user
     * @return  Illuminate\Database\Eloquent\Collection
     */
    public function getUserActions(UserRepository $user)
    {
        return $this->select('manager_pages.title as pageTitle', 
                             'manager_pages.slug as pageSlug', 
                             'manager_page_actions.title as actionTitle', 
                             'manager_page_actions.type as actionType')
             ->joinUser()
             ->joinPage()
             ->joinAction()
             ->where('manager_users.id', '=', $user->id)
             ->get();
    }
}